@@ -1,16 +1,16 @@
 from __future__ import absolute_import
-
 import logging
 from xml.etree import ElementTree
-import requests
 
+import requests
 from lexicon.providers.base import Provider as BaseProvider
 
-logger = logging.getLogger(__name__)
+
+LOGGER = logging.getLogger(__name__)
 
 APIENTRYPOINT = {
-        'zonomi': 'https://zonomi.com/app',
-        'rimuhosting' : 'https://rimuhosting.com'
+    'zonomi': 'https://zonomi.com/app',
+    'rimuhosting': 'https://rimuhosting.com'
 }
 
 NAMESERVER_DOMAINS = ['zonomi.com']
@@ -35,43 +35,43 @@ NAMESERVER_DOMAINS = ['zonomi.com']
 
 
 def ProviderParser(subparser):
-    subparser.add_argument("--auth-token", help="specify token for authentication")
+    subparser.add_argument(
+        "--auth-token", help="specify token for authentication")
     subparser.add_argument("--auth-entrypoint", help="use Zonomi or Rimuhosting API", choices=[
-        'zonomi', 'rimuhosting' ])
+        'zonomi', 'rimuhosting'])
 
 
 class Provider(BaseProvider):
 
-    def __init__(self, options, engine_overrides=None):
-        super(Provider, self).__init__(options, engine_overrides)
+    def __init__(self, config):
+        super(Provider, self).__init__(config)
         self.domain_id = None
-        if not self.options.get('auth_token'):
+        if not self._get_provider_option('auth_token'):
             raise Exception('Error, application key is not defined')
 
-        self.api_endpoint = self.engine_overrides.get('api_endpoint', APIENTRYPOINT.get('zonomi'))
-
-        if self.options.get('auth_entrypoint'):
-            self.api_endpoint = self.engine_overrides.get('api_endpoint', APIENTRYPOINT.get(self.options.get('auth_entrypoint')))
+        self.api_endpoint = APIENTRYPOINT.get('zonomi')
 
+        if self._get_provider_option('auth_entrypoint'):
+            self.api_endpoint = APIENTRYPOINT.get(
+                self._get_provider_option('auth_entrypoint'))
 
     def authenticate(self):
-    
+
         payload = self._get('/dns/dyndns.jsp', {
-            'action' : 'QUERY',
-            'name' : "**." + self.options['domain'],
-            'type' : 'SOA'
+            'action': 'QUERY',
+            'name': "**." + self.domain,
+            'type': 'SOA'
         })
 
         if payload.find('is_ok').text != 'OK:':
-            raise Exception('Error with api {0}'.format(payload.find('is_ok').text))
-
-        self.domain_id = self.options['domain']
+            raise Exception('Error with api {0}'.format(
+                payload.find('is_ok').text))
 
+        self.domain_id = self.domain
 
     def _make_identifier(self, type, name, content):
         return "{}/{}={}".format(type, self._full_name(name), content)
 
-
     def _parse_identifier(self, identifier):
         parts = identifier.split('/')
         type = parts[0]
@@ -80,39 +80,38 @@ class Provider(BaseProvider):
         content = "=".join(parts[1:])
         return type, name, content
 
-
     def create_record(self, type, name, content):
         request = {
-                'action': 'SET', 
-                'type': type, 
-                'name': self.options['domain'], 
-                'value': content 
+            'action': 'SET',
+            'type': type,
+            'name': self.domain,
+            'value': content
         }
 
         if name is not None:
             request['name'] = self._full_name(name)
 
-        if self.options.get('ttl'):
-            request['ttl'] = self.options.get('ttl')
+        if self._get_lexicon_option('ttl'):
+            request['ttl'] = self._get_lexicon_option('ttl')
 
-        if self.options.get('priority'):
-            request['prio'] = self.options.get('priority')
+        if self._get_lexicon_option('priority'):
+            request['prio'] = self._get_lexicon_option('priority')
 
         payload = self._get('/dns/dyndns.jsp', request)
 
         if payload.find('is_ok').text != 'OK:':
-            raise Exception('An error occurred: {0}'.format(payload.find('is_ok').text))
+            raise Exception('An error occurred: {0}'.format(
+                payload.find('is_ok').text))
 
-        logger.debug('create_record: %s', True)
+        LOGGER.debug('create_record: %s', True)
         return True
 
-
     def list_records(self, type=None, name=None, content=None):
         records = []
 
         request = {
-            'action' : 'QUERY',
-            'name': "**." + self.options['domain']
+            'action': 'QUERY',
+            'name': "**." + self.domain
         }
 
         if type is not None:
@@ -128,23 +127,22 @@ class Provider(BaseProvider):
                 'type': rxml.attrib['type'],
                 'name': rxml.attrib['name'],
                 'content': rxml.attrib['content'],
-                'id': self._make_identifier(rxml.attrib['type'],rxml.attrib['name'],rxml.attrib['content']),
+                'id': self._make_identifier(rxml.attrib['type'], rxml.attrib['name'], rxml.attrib['content']),
                 'ttl': rxml.attrib['ttl'].split()[0]
             }
             records.append(processed_record)
-        logger.debug('list_records: %s', records)
+        LOGGER.debug('list_records: %s', records)
         return records
 
-
     def delete_record(self, identifier=None, type=None, name=None, content=None):
         if identifier is not None:
             type, name, content = self._parse_identifier(identifier)
 
         request = {
-            'action' : 'DELETE',
-            'name': self.options['domain'] 
+            'action': 'DELETE',
+            'name': self.domain
         }
-        
+
         if type is not None:
             request['type'] = type
         if name is not None:
@@ -155,20 +153,20 @@ class Provider(BaseProvider):
         payload = self._get('/dns/dyndns.jsp', request)
 
         if payload.find('is_ok').text != 'OK:':
-            raise Exception('An error occurred: {0}'.format(payload.find('is_ok').text))
+            raise Exception('An error occurred: {0}'.format(
+                payload.find('is_ok').text))
 
-        logger.debug('delete_record: %s', True)
+        LOGGER.debug('delete_record: %s', True)
         return True
 
-
     def update_record(self, identifier, type=None, name=None, content=None):
         self.delete_record(identifier)
         ttype, tname, tcontent = self._parse_identifier(identifier)
         request = {
-                'action': 'SET',
-                'type': ttype,
-                'name': self._full_name(tname),
-                'value': tcontent
+            'action': 'SET',
+            'type': ttype,
+            'name': self._full_name(tname),
+            'value': tcontent
         }
 
         if type is not None:
@@ -177,17 +175,18 @@ class Provider(BaseProvider):
             request['name'] = self._full_name(name)
         if content is not None:
             request['value'] = content
-        if self.options.get('ttl'):
-            request['ttl'] = self.options.get('ttl')
-        if self.options.get('priority'):
-            request['prio'] = self.options.get('priority')
+        if self._get_lexicon_option('ttl'):
+            request['ttl'] = self._get_lexicon_option('ttl')
+        if self._get_lexicon_option('priority'):
+            request['prio'] = self._get_lexicon_option('priority')
 
         payload = self._get('/dns/dyndns.jsp', request)
 
         if payload.find('is_ok').text != 'OK:':
-            raise Exception('An error occurred: {0}'.format(payload.find('is_ok').text))
+            raise Exception('An error occurred: {0}'.format(
+                payload.find('is_ok').text))
 
-        logger.debug('update_record: %s', True)
+        LOGGER.debug('update_record: %s', True)
         return True
 
     def _request(self, action='GET', url='/', data=None, query_params=None):
@@ -196,9 +195,10 @@ class Provider(BaseProvider):
         if query_params is None:
             query_params = {}
         else:
-            query_params['api_key'] = self.options.get('auth_token')
+            query_params['api_key'] = self._get_provider_option('auth_token')
 
-        r = requests.request(action, self.api_endpoint + url, params=query_params)
+        r = requests.request(action, self.api_endpoint +
+                             url, params=query_params)
         tree = ElementTree.ElementTree(ElementTree.fromstring(r.content))
         root = tree.getroot()
         if root.tag == 'error':
